package com.factory.wx.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.fa.FirstArticleApprovalTask;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.res.ApprovalUserRes;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleApprovalTaskMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.fa.*;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.req.PlanPassReq;
import com.factory.wx.entity.req.SearchPlanPassReq;
import com.factory.wx.entity.res.*;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.AbnormalStatisticsService;
import com.factory.wx.service.PlanPassService;
import com.factory.wx.service.WxPlanService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Service("planPassService")
public class PlanPassServiceImpl implements PlanPassService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanPassServiceImpl.class);
    /**
     * 审批规则类型为：2首件
     */
    private static int APPROVAL_TYPE_FIRST_ARTICLE = 2;
    /**
     * 审批规则类型为：3防错
     */
    private static int APPROVAL_TYPE_PM = 3;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private FirstArticleApprovalTaskMapper firstArticleApprovalTaskMapper;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private FirstArticleItemService firstArticleItemService;
    @Autowired
    private FirstArticleItemResultService firstArticleItemResultService;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private FirstArticleTaskService firstArticleTaskService;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private AbnormalStatisticsService abnormalStatisticsService;
    @Autowired
    private OnsiteWorkshopService onsiteWorkshopService;

    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
//    @Override
    public List<PlanPassRes> getPassList(PlanPageReq planPageReq) {
        //分页
        planPageReq.setPage((planPageReq.getPage() - 1) * planPageReq.getSize());
        List<PlanPassRes> planPassResList = new ArrayList<>();
        //首件列表
        if (planPageReq.getCheckType() == 1) {
            planPassResList = planPassMapper.getPlanPassList(planPageReq);
        }
        //防错列表
        if (planPageReq.getCheckType() == 2) {
            planPassResList = planPassMapper.getPlanPassPMList(planPageReq);
        }
        return planPassResList;
    }

    /**
     * 计划放行详情-防错
     *
     * @param planPassReq
     * @return
     */

    @Override
    public PlanPassDetailRes getPassDetailPM(PlanPassReq planPassReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getOnsitePlanBaseInfo(planPassReq.getPlanId());

        List<PlanPassTaskRes> planPassTaskResList = getPMTaskList(planPassReq.getPlanId());

        List<Integer> itemType = getItemFrequency(planPassTaskResList);

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .taskList(planPassTaskResList)
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .build();
    }

    /**
     * 获取防错点检错误列表
     *
     * @param planId
     * @return
     */
    private List<PlanPassTaskRes> getPMTaskList(Long planId) {
        List<PlanPassTaskRes> pmList = planPassMapper.getPmPassTaskListByPlanId(planId);
        for (PlanPassTaskRes pmInfo : pmList) {
            List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                    .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getPlanId()));

            List<PlanPassWorkshopRes> workshopList = new ArrayList<PlanPassWorkshopRes>();
            for (OnsiteWorkshop workshop : onsiteWorkshopList) {
                PlanPassWorkshopRes planPassWorkshopRes = new PlanPassWorkshopRes();
                planPassWorkshopRes.setWorkshopId(workshop.getId());
                planPassWorkshopRes.setWsName(workshop.getWsName());
                workshopList.add(planPassWorkshopRes);
            }

            for (int i = 0; i < workshopList.size(); i++) {
                SearchPlanPassReq searchReq = new SearchPlanPassReq();
                searchReq.setWorkshopId(workshopList.get(i).getWorkshopId());
                searchReq.setTaskId(pmInfo.getTaskId());
                List<PlanPassItemRes> planPassItemList = planPassMapper.getPmPassItemList(searchReq).stream().filter(item -> item.getResult() > 0).collect(Collectors.toList());
                if (planPassItemList.size() == 0) {
                    workshopList.remove(i);
                } else {
                    for (PlanPassItemRes planPassItemRes : planPassItemList) {
                        Long userId = planPassItemRes.getItemResultUserId();
                        if (null != userId) {
                            String userName = wxPlanService.getUserNameById(planPassItemRes.getItemResultUserId());
                            planPassItemRes.setItemResultUser(userName);
                        }
                    }
                    workshopList.get(i).setItemList(planPassItemList);
                }
            }
            pmInfo.setWorkshopList(workshopList);
        }
        return pmList;
    }

    /**
     * 获取防错频次
     *
     * @param planPassTaskResList
     */
    private List<Integer> getItemFrequency(List<PlanPassTaskRes> planPassTaskResList) {
        List<Integer> frequency = new ArrayList<>();
        planPassTaskResList.forEach(task -> {
            task.getWorkshopList().forEach(workshop -> {
                workshop.getItemList().forEach(item -> {
                    frequency.add(item.getItemType());
                });
            });
        });
        return frequency.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 计划放行详情-首件
     *
     * @param planPassReq
     * @return
     */
    @Override
    public PlanPassDetailRes getPassDetail(PlanPassReq planPassReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getFirstArticlePlanBaseInfo(planPassReq.getPlanId());

        List<PlanPassTaskRes> planPassTaskResList = getFATaskList(planPassReq.getPlanId());

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(wxPlanBaseInfoRes.getFrequency())
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .taskList(planPassTaskResList)
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .build();
    }

    private List<PlanPassTaskRes> getFATaskList(Long planId) {
        List<PlanPassTaskRes> faList = planPassMapper.getFaPassTaskListByPlanId(planId);

        for (PlanPassTaskRes faInfo : faList) {
            List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                    .eq(FirstArticleWorkshop::getBentelerPlanId, planId));

            List<PlanPassWorkshopRes> planPassWorkshopList = new ArrayList<PlanPassWorkshopRes>();
            PlanPassWorkshopRes planPassWorkshop = null;
            for (FirstArticleWorkshop workshop : firstArticleWorkshopList) {
                planPassWorkshop = new PlanPassWorkshopRes();
                planPassWorkshop.setWorkshopId(workshop.getId());
                planPassWorkshop.setWsName(workshop.getWsName());
                planPassWorkshop.setPlName(workshop.getPlName());
                planPassWorkshopList.add(planPassWorkshop);
            }

            for (int i = 0; i < planPassWorkshopList.size(); i++) {

                SearchPlanPassReq searchReq = new SearchPlanPassReq();
                searchReq.setWorkshopId(planPassWorkshopList.get(i).getWorkshopId());
                List<PlanPassItemRes> planPassItemList = planPassMapper.getFaPassItemList(searchReq);

                for (PlanPassItemRes planPassItemRes : planPassItemList) {
                    List<PlanPassItemResultRes> firstItemResultList = this.getFaItemResultList(faInfo.getTaskId(), planPassItemRes.getItemId(), 0);
                    planPassItemRes.setFirstItemResultList(firstItemResultList);

                    List<PlanPassItemResultRes> twoHundredItemResultList = this.getFaItemResultList(faInfo.getTaskId(), planPassItemRes.getItemId(), 1);
                    planPassItemRes.setTwoHundredItemResultList(twoHundredItemResultList);

                    List<PlanPassItemResultRes> catchItemResultList = this.getFaItemResultList(faInfo.getTaskId(), planPassItemRes.getItemId(), 2);
                    planPassItemRes.setCatchItemResultList(catchItemResultList);
                }
                if (planPassItemList.size() == 0) {
                    planPassWorkshopList.remove(i);
                } else {
                    planPassWorkshopList.get(i).setItemList(planPassItemList);
                }

                faInfo.setWorkshopList(planPassWorkshopList);
            }
        }
        return faList;
    }

    private List<PlanPassItemResultRes> getFaItemResultList(long taskId, long itemId, int typeFlag) {
        SearchPlanPassReq searchReq = new SearchPlanPassReq();
        searchReq.setTaskId(taskId);
        searchReq.setItemId(itemId);
        searchReq.setTypeFlag(typeFlag);
        List<PlanPassItemResultRes> planPassItemResultResList = planPassMapper.getFaPassItemResultList(searchReq);
        return planPassItemResultResList;
    }
    /**
     * 计划放行详情-首件
     *
     * @param planPassReq
     * @return
     */


    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean approvalTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {

        int updateStatus = 0; // 更新状态
        int taskStatus = 0; //task表状态
        if (planApprovalTaskReq.getStatus() == 1) taskStatus = 2; // 放行
        if (planApprovalTaskReq.getStatus() == 2) taskStatus = 3; // 不通过

        if (planApprovalTaskReq.getPlanType() == 1) {
            //更新首件审批信息
            updateStatus = updateFirstArticleApprovalTask(planApprovalTaskReq, taskStatus);
        }
        if (planApprovalTaskReq.getPlanType() == 2) {
            //更新防错审批信息
            updateStatus = updateOnsiteApprovalTask(planApprovalTaskReq, taskStatus);
        }

        if (updateStatus > 0) {
            settingApprovalTask(planApprovalTaskReq);
        }

        return ResponseBeanBuilder.ok();
    }

    /**
     * 更新首件任务审批状态
     *
     * @param planApprovalTaskReq
     * @param taskStatus
     * @return
     */
    private int updateFirstArticleApprovalTask(PlanApprovalTaskReq planApprovalTaskReq, int taskStatus) {
        int updateStatus;
        FirstArticleApprovalTask ot = new FirstArticleApprovalTask();
        ot.setFirstArticleTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        updateStatus = firstArticleApprovalTaskMapper.update(ot, Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                .eq(FirstArticleApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        //更新task表状态
        firstArticleTaskService.updateById(FirstArticleTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
        return updateStatus;
    }

    /**
     * 更新防错任务审批状态
     *
     * @param planApprovalTaskReq
     * @param taskStatus
     * @return
     */
    private int updateOnsiteApprovalTask(PlanApprovalTaskReq planApprovalTaskReq, int taskStatus) {
        int updateStatus;
        OnsiteApprovalTask ot = new OnsiteApprovalTask();
        ot.setOnsiteTaskId(planApprovalTaskReq.getTaskId());
        ot.setReason(planApprovalTaskReq.getReason());
        ot.setStatus(planApprovalTaskReq.getStatus());
        updateStatus = onsiteApprovalTaskMapper.update(ot, Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()));
        //更新task表状态
        onsiteTaskMapper.updateById(OnsiteTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
        return updateStatus;
    }

    /**
     * 设置审批task任务
     *
     * @param planApprovalTaskReq
     */
    private void settingApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        //审批不通过触发下一级别
        if (planApprovalTaskReq.getStatus() == 2) {
            //设置首件下一级别审批
            if (planApprovalTaskReq.getPlanType() == 1) {
                setNextApprovalLevel(planApprovalTaskReq);
            }
            //设置防错下一级别审批
            if (planApprovalTaskReq.getPlanType() == 2) {
                setNextPMApprovalLevel(planApprovalTaskReq);
            }
        }
        //审批通过，保存审批历史
        if (planApprovalTaskReq.getStatus() == 1) {
            //首件
            if (planApprovalTaskReq.getPlanType() == 1) {
                firstArticleService.setFirstArticleTaskHis(planApprovalTaskReq.getTaskId());
            }
            //防错
            if (planApprovalTaskReq.getPlanType() == 2) {
                preventMistakeService.setOnsiteTaskHis(planApprovalTaskReq.getTaskId());
            }
        }
    }

    /**
     * 设置防错下一级别人员
     *
     * @param planApprovalTaskReq
     */
    private void setNextPMApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, APPROVAL_TYPE_PM)
                .orderByDesc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        //获取当前审批级别
        OnsiteApprovalTask onsiteApprovalTask = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
        int userLevel = onsiteApprovalTask.getApprovalLevel();
        if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
            onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                    .onsiteTaskId(onsiteApprovalTask.getOnsiteTaskId())
                    .approvalLevel(approvalUserRes.getApprovalLevel())
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0)
                    .build());
        }
    }

    /**
     * 设置首件下一级别人员
     *
     * @param planApprovalTaskReq
     */
    private void setNextApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleMaxLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, APPROVAL_TYPE_FIRST_ARTICLE)
                .orderByDesc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        //获取当前审批级别
        FirstArticleApprovalTask firstArticleApprovalTask = firstArticleApprovalTaskMapper.selectOne(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                .eq(FirstArticleApprovalTask::getApprovalUserId, planApprovalTaskReq.getUserId()).last("limit 1"));
        int userLevel = firstArticleApprovalTask.getApprovalLevel();
        if (userLevel < approvalRuleMaxLevel.getApprovalLevel()) {
            ApprovalUserRes approvalUserRes = preventMistakeService.getApprovalUserInfo(userLevel + 1, APPROVAL_TYPE_PM);
            onsiteApprovalTaskMapper.insert(OnsiteApprovalTask.builder()
                    .onsiteTaskId(firstArticleApprovalTask.getFirstArticleTaskId())
                    .approvalLevel(approvalUserRes.getApprovalLevel())
                    .approvalUserId(approvalUserRes.getUserId())
                    .status(0)
                    .build());
        }
    }

}
